/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xalan" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, Lotus
 * Development Corporation., http://www.lotus.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.xalan.xslt;

import org.w3c.dom.*;
import org.xml.sax.*;
import org.apache.xalan.xpath.*;
import org.apache.xalan.xpath.xml.QName;
import java.util.StringTokenizer;

/**
 * <meta name="usage" content="advanced"/>
 * Implement xsl:use.
 * This acts as a superclass for ElemCopy, ElemAttributeSet,
 * ElemElement, and ElemLiteralResult, on order to implement 
 * shared behavior the use-attribute-sets attribute.
 */
public class ElemUse extends ElemTemplateElement
{
  public QName m_attributeSetsNames[] = null;

  public ElemUse (XSLTEngineImpl processor,
                  Stylesheet stylesheetTree,
                  String name, 
                  AttributeList atts,
                  int lineNumber, int columnNumber)
    throws SAXException
  {
    super(processor, stylesheetTree, name, atts, lineNumber, columnNumber);
  }
  
  public void execute(XSLTEngineImpl processor, 
                      Node sourceTree, 
                      Node sourceNode,
                      QName mode)
    throws XSLProcessorException, 
           java.net.MalformedURLException, 
           java.io.FileNotFoundException, 
           java.io.IOException,
           SAXException
  {
    super.execute(processor, sourceTree, sourceNode, mode);
    if(null != m_attributeSetsNames)
    {
      m_stylesheet.applyAttrSets(m_attributeSetsNames, 
                                 processor, sourceTree, sourceNode, mode);
    }
  }

  /** 
   * See if this is a use-attribute-sets attribute, and, if so, process it.
   * 
   * @param attrName Qualified name of attribute.
   * @param atts The attribute list where the element comes from (not used at 
   *      this time).
   * @param which The index into the attribute list (not used at this time).
   * @return True if this is a use-attribute-sets attribute.
   */
  boolean processUseAttributeSets(String attrName, AttributeList atts, int which)
  {
    boolean isUAS = false;
    if(Constants.ELEMNAME_LITERALRESULT == getXSLToken())
    {
      QName qname = new QName(attrName, m_stylesheet.m_namespaces);
      isUAS = ((null != qname.m_namespace) &&
               (qname.m_namespace.equals(m_stylesheet.m_XSLNameSpaceURL)) &&
               (qname.m_localpart.equals(Constants.ATTRNAME_USEATTRIBUTESETS)));
    }
    else
    {
      isUAS = attrName.equals(Constants.ATTRNAME_USEATTRIBUTESETS);
    }
    if(isUAS)
    {
      isUAS = true;
      String qnames = atts.getValue(which);
      StringTokenizer tokenizer = new StringTokenizer(qnames, " \t\n\r", false);
      m_attributeSetsNames = new QName[tokenizer.countTokens()];
      for(int k = 0; tokenizer.hasMoreTokens(); k++)
      {
        m_attributeSetsNames[k] = new QName(tokenizer.nextToken(), m_stylesheet.m_namespaces);
      }
    }
    return isUAS;
  }
}
